#################### FILE ABSTRACT ############################################
##
## Synopsis::         Source for common routines
##
## File Name::        %M%
## Version::          %I%
## Type::             source
## Date Completed::   %G%
##
## Environment::
##     Language::     Splus
##     Computer::     Compaq/Alpha
##     OS::           DigitalUNIX
##     OSVersion::    4.0
##
## Module Information::
##     Owner::        MDACC
##     Department::   Biostatistics
##     Researcher::   Kim-Anh Do
##     Application::  geneclust
##
###############################################################################


if (GCinterpreter == "R") {

	AsciiToInt <- function(s){return(as.integer(s));}
	get.message <- function() { return ("unknown"); }
	dump.calls <- traceback;
	q <- function() { return (quit ("no")); }
}

##*****************************************************************************
GetAbsPathName.func<-function(pathname)
{
    ## Test if path is relative or absolute
    secchar<-substring(pathname,2,2);
    if (secchar == ":")
        abspathname<-pathname
    else
        abspathname<-paste(GChome,pathname,sep="/");

    return(abspathname);
}


##******************************************************************************
PlotGap.func<-function(MyGapFile)
{
    MyFrame<-read.table(MyGapFile,header=FALSE);
    x<-MyFrame$V1;
    y<-MyFrame$V2;

    plot(x,y,col=1,pch=5,log="x",
         xlab="Cluster Size",ylab="Gap Curve");

    nperm <- ncol (MyFrame) - 2;
    for (i in 1:nperm)
        points(x,MyFrame[,i+2],col=3,type="l");

    return();
}


##******************************************************************************
PlotPC.func<-function(MyPCFile)
{
    MyFrame<-read.table(MyPCFile,header=FALSE);
    y<-MyFrame$V1;
    x<-c(1:length(y));

    plot(x,y,col=1,pch=5,
         xlab="Sample",ylab="Principal Component");

    return();
}


##*****************************************************************************
PlotVariance.func<-function(MyVarFile)
{
    MyFrame<-read.table(MyVarFile,header=FALSE);
    x<-MyFrame$V1;
    y<-MyFrame$V2;
    z<-MyFrame$V3;

    plot(x,y,col=1,pch=0,log="x",mkh=0.1,
         xlab="Cluster Size",ylab="Variance Ratio",ylim=c(0,100));
    points(x,z,col=2,pch=1,mkh=0.1);

    nperm <- ncol (MyFrame) - 6;
    for (i in 1:nperm) {
       points(x,MyFrame[,i+6],col=3,type="l");
    }
    return();
}


##******************************************************************************
CheckNoise.func<-function(X,noise,df)
{
    N<-dim(X)[1];
    p<-dim(X)[2];

    if (noise == 0) {
        ## No noise
        Z<-X;
        Y<-c(rep(0,N*p));
    }
    else if (noise == 1) {
        ## Generate Gaussian noise
        v<-rnorm(N*p, mean=0, sd=1)
        Y<-matrix(v,N,ncol=p);
        Z<-X + Y;
    }
    else if (noise == 2) {
        ## Generate Chi-square noise
        v<-rchisq(N*p,df);
        Y<-matrix(v,N,ncol=p);
        Z<-X + Y;
    }
    else if (noise == 3) {
        ## Generate t noise (Student's distribution)
        v<-rt(N*p,df);
        Y<-matrix(v,N,ncol=p);
        Z<-X + Y;
    }

    return(Z,Y);
}


##******************************************************************************
Hclust.func<-function(K,nclst,identifier,fname)
{
    basetitle<-paste("Hierarchical Clustering of",identifier,sep=" ");
    dd<-dist(K,method="euclidean");
    htemp<-hclust(dd,method="complete");
    cnames<-c(1:dim(K)[1]);

    Motif(paste("SPLUS",basetitle,sep=" : "));
    # Print the hierarchical clustering graph
    #hclstgraph<-plclust(htemp,unit=TRUE,level=TRUE,labels=cnames);
   #hclstgraph<-plclust(htemp,labels=cnames,xlab=identifier,sub="");
   hclstgraph<-plot(htemp,labels=cnames,cex=0.65,main="",xlab="",sub="");
    maintitle<-paste(basetitle,"\n",sep="");
    title(maintitle);
    outgraph<-paste(GChome,'output',fname,sep="/");

    if (GCinterpreter == "R") {
   	if (psout != 0) dev.copy2eps(file=outgraph);
    }
    else {
        printgraph(horizontal=TRUE,print=FALSE,file=outgraph);
    }

    return();
}


##******************************************************************************
DisplayHclust.func<-function(K,nclst)
{
    if (graphGenes || graphSamples) {
        cat("PK_MONITOR_HIERARCHICAL_CLUSTERING\n");

        ## Cluster genes
        if (graphGenes)
            Hclust.func(K,nclst,"Genes","GChclustgenes.ps");

        ## Cluster samples
        if (graphSamples)
            Hclust.func(t(K),nclst,"Samples","GChclustsamples.ps");
    }

    return();
}


##******************************************************************************
ShowMatrix.func<-function(X,N,p)
{
    rowlblsfile<-paste(GCoutput,"RowLbls.txt",sep="/");
    collblsfile<-paste(GCoutput,"ColLbls.txt",sep="/");
    matrixfile <-paste(GCoutput,"GCMatrix.txt",sep="/");

X[is.na(X)]<-mean(X[!is.na(X)]);
    k<-matrix(X,nrow=N,dimnames=list(scan(rowlblsfile,what="",n=N,sep="\n"),
                                     scan(collblsfile,what="",n=p,sep="\n")));
    x<-c(1:N);
    y<-c(1:p);
    Motif(paste("SPLUS","Gene Expression Matrix",sep=" : "));

    if (GCinterpreter == "R")
    {
        matrixgraph<-image(x,y,k,axes=FALSE,col=ColorSchemeImages.func(),xlab="",ylab="");
        legend(k,x=1.045*N,y=p,hor=F,cex=0.65);
        par (cex.axis=0.6,las=2);   #Here is the size of labels for the 1st image

labels1 <- dimnames(k)[[1]];
labels2 <- dimnames(k)[[2]];

charH <- par('cin')[2];
Lx <- par('pin')[1];
Ly <- par('pin')[2];
maxX <- Lx/charH;
maxY <- Ly/charH;

xold <- x;
yold <- y;

#Take care of overlapping labels
	if (p>maxY){tmp1 <- as.integer(p/maxY);
		    y<-seq(1,p,tmp1);
			labels2 <- labels2[y];
 			 }
        if (N>maxX){tmp1 <- as.integer(N/maxX);
		    x<-seq(1,N,tmp1);
			labels1 <- labels1[x]; }

       axis(1,at=x,labels=labels1,adj=1,col=1);
        axis(2,at=y,labels=labels2,adj=1,col=1);

	par(cex.axis=1,las=1);


    }
    else {
        matrixgraph<-image(x,y,k,axes=FALSE);
        image.legend(k,x=1.045*N,y=p,size=c(0.085,4.2),hor=F,cex=0.65);
        axis(1,at=x,labels=dimnames(k)[[1]],adj=1,col=1,cex=0.45,srt=90);
        axis(2,at=y,labels=dimnames(k)[[2]],adj=1,col=1,cex=0.45);
    }
    ## Save matrix to a text file/data frame
    write.table(X,file=matrixfile,sep="\t");
    #print(matrixgraph);
    return();
}


##******************************************************************************
## npar - number of screen partitions
##******************************************************************************
Show.func<-function(N,p,numclusters,npar)
{
    ## Shows all clusters on the same graph
    n<-1;
    index1<-1;
    w<-8.15;
    h<-2.5;
    rows<-2;
    cols<-1;
    basetitle<-"GeneShave Cluster Plots";

    ## Open "vlabels.txt" and extract the variance filenames
    varlabelsfile<-paste(GCoutput,"vlabels.txt",sep="/");
    varlabels<-matrix(scan(varlabelsfile,what="",n=numclusters,sep="\n"),
                           numclusters,byrow=FALSE);
    ## Open "clabels.txt" file and extract the cluster file names
    labelsfile<-paste(GCoutput,"clabels.txt",sep="/");
    genelabels<-matrix(scan(labelsfile,what="",n=numclusters,sep="\n"),
                            numclusters,byrow=FALSE);
    ## Open "elabels.txt" file and extract the eigenvalue file names
    elabelsfile<-paste(GCoutput,"elabels.txt",sep="/");
    elabels<-matrix(scan(elabelsfile,what="",n=numclusters,sep="\n"),
                         numclusters,byrow=FALSE);
    ## Open "pclabels.txt" file and extract the pc vector file names
    pclabelsfile<-paste(GCoutput,"pclabels.txt",sep="/");
    pclabels<-matrix(scan(pclabelsfile,what="",n=numclusters,sep="\n"),
                          numclusters,byrow=FALSE);
    
    ## Open "clusterlength.txt" file and extract a matrix with first
    ## column optimal cluster index and second column number of genes
    ## per cluster
    clusterlengthfile<-paste(GCoutput,"clusterlengths.txt",sep="/");
    Ncluster<-matrix(scan(clusterlengthfile),numclusters,byrow=TRUE);
    ## Open "clusters.txt" file and extract cluster filenames from it
    clustersfile<-paste(GCoutput,"clusters.txt",sep="/");
    stringmatrix<-matrix(scan(clustersfile,what="",
                              n=numclusters,sep="\n"),
                         numclusters,byrow=FALSE);
    ## Generate Column labels filename with proper path
    columnfile<-paste(GCoutput,"ColLbls.txt",sep="/");

    while (n <= numclusters) {
        flag<-n%%npar;
        if (flag == 1) {
            Motif(paste("SPLUS",basetitle,sep=" : "));
            par(mfrow=c(npar,1));
        }
        myevalue<-paste(GCoutput,elabels[n,1],sep="/");
        evalue<-read.table(file=myevalue,header=FALSE);
        varlabels[n]<-paste(GCoutput,varlabels[n],sep="/");
        variance<-read.table(file=varlabels[n],header=FALSE);
        genelabels[n]<-paste(GCoutput,genelabels[n],sep="/");
        stringmatrix[n]<-paste(GCoutput,stringmatrix[n],sep="/");
        k<-matrix(scan(file=stringmatrix[n]),ncol=p,byrow=FALSE);
        k<-matrix(k,nrow=nrow(k),dimnames=list(scan(genelabels[n],what="",
                  n=Ncluster[n,2],sep="\n"),scan(columnfile,what="",
                  n=p,sep="\n")));
        pclabelfile<-paste(GCoutput,pclabels[n,1],sep="/");
        pcvalue<-scan(file=pclabelfile);
        weights<-rep(0,Ncluster[n,2]);
        for (i in 1:Ncluster[n,2]) {
            weights[i]<-sum(k[i,]*pcvalue);
        }

        x<-c(1:Ncluster[n,2]);
        y<-c(1:p);
        ## Display image of cluster sorted w.r.t. column means
        me<-c(rep(0,p));
        for (i in 1:p) {
            me[i]<-mean(k[,i]);
        }
        k1<-k[sort.list(weights),sort.list(me)];

        xloc<-length(dimnames(k1)[[2]]);
        yloc<-length(dimnames(k1)[[1]]);
        if (GCinterpreter == "R") {
            image(t(k1),axes=FALSE,col=ColorSchemeImages.func());
            legend(k1,x=1.045*xloc,y=yloc,hor=F,cex=0.65);
	}
	else {
            image(t(k1),axes=FALSE);
            image.legend(k1,x=1.045*xloc,y=yloc,size=c(0.085,1.55),hor=F,cex=0.65);
	}
        ## To include axes titles do:
        ##     image(x,y,k,xlab="GENES",ylab="Cell Boundaries");
        ##
        ## The col par sets the colcor of the axis and text. Since I'd like my
        ## background to be dark blue I set it = 0
        ##
        ## cex = .5 defines size of the font used in labels (height in inches)

labels1 <- dimnames(k1)[[2]];
labels2 <- dimnames(k1)[[1]];


charH <- par('cin')[2];
Lx <- par('pin')[1];
Ly <- par('pin')[2];
maxX <- Lx/charH;
maxY <- Ly/charH;


xold <- x;
yold <- y;


if (length(y)>maxX) { tmp1 = as.integer(length(y)/maxX);
			len1<-length(y);
		      y <- y[seq(1,len1,tmp1)];
#axis(1,at=(yold-1)/(p-1),labels=FALSE);
		       labels1 <- labels1[seq(1,len1,tmp1)]; };
#print(length(labels1));

if (length(x)>maxY) { tmp1 = as.integer(length(x)/maxY);
		      len1<-length(x);
		      x <- x[seq(1,len1,tmp1)];
#axis(2,at=(xold-1)/(Ncluster[n,2]-1),labels=FALSE);
		      labels2 <- labels2[seq(1,len1,tmp1)]; };
		      	

        if (GCinterpreter == "R") {
	    par (cex.axis=0.6,las=2);
            axis(2,at=(x-1)/(Ncluster[n,2]-1),labels=labels2, adj=1, col= 1, cex=0.2,tick=TRUE);
            axis(1,at=(y-1)/(p-1),labels=labels1, adj= 1, col= 1, cex=0.2,tick=TRUE);

	    box();
	    par(cex.axis=1,las=1);
        }
        else {
            axis(2,at=x,labels=dimnames(k1)[[1]], adj=1, col= 1, cex= .5);
            axis(1,at=y,labels=dimnames(k1)[[2]], srt=90, adj= 1, col= 1, cex= .5);
        }
        if (flag == 1) {
            maintitle<-paste(basetitle,"\n",sep="");
            title(main=maintitle);
        }
        else {
            maintitle<-"";
        }
       
        optindex<-Ncluster[n,1];
        percvariance<-variance[optindex,2];
        tvariance<-variance[optindex,5];
        bvariance<-variance[optindex,4];
        relvariance<-variance[optindex,4]/variance[optindex,6];
        clustertitle<-paste(maintitle,"Cluster #",n);
        clustersubtitle<-paste(" eigenvalue=",round(evalue,4),
                               " %variance=",round(percvariance,4),
                               " VT=",round(tvariance,4),
                               " VB=",round(bvariance,4),
                               " VB/VW=",round(relvariance,4));
        title(main=clustertitle,sub=clustersubtitle);

        if ((n%%2) == 0 || n == numclusters) {
            outfile<-paste(GChome,'output',"GCclusterplots",sep="/");
            outfile<-paste(outfile,index1,".ps",sep="");
            if (GCinterpreter == "R") {
                 if (psout != 0)  dev.copy2eps(file=outfile)
			
            }
            else {
            printgraph(horizontal=TRUE,print=FALSE,file=outfile);
            }
            index1<-index1+1;
        }
        n<-n+1;
    }
}


##******************************************************************************
ShowGap.func<-function(numclusters)
{
    ## Shows all clusters on the same graph
    n<-1;
    index1<-1;
    npar<-4;
    basetitle<-"GeneShave Gap Curve Graphs";
    gaplabelfile<-paste(GCoutput,"glabels.txt",sep="/");
    gaplabels<-matrix(scan(gaplabelfile,what="", n=numclusters,sep="\n"),
                      numclusters,byrow=FALSE);

    while (n <= numclusters) {
        flag<-n%%npar;
        if (flag == 1) {
            Motif(paste("SPLUS",basetitle,sep=" : "));
            par(mfrow=c(2,2),oma=c(1,1,1,1));
        }  

        gapfile<-paste(GCoutput,gaplabels[n,1],sep="/");
        PlotGap.func(gapfile);
        clustertitle<-paste("\n","Cluster #",n);
        title(main=clustertitle,cex=1.0);
        if ((n%%npar) == 0 || n == numclusters) {
            maintitle<-paste("\n",basetitle,sep="");
            mtext(maintitle,side=3,outer=TRUE,cex=1.1,font=2);
            ## Save graphs to postscript file
            outfile<-paste(GChome,'output',"GCgapplots",sep="/");
            outfile<-paste(outfile,index1,".ps",sep="");
            if (GCinterpreter == "R") {
             if (psout != 0) dev.copy2eps(file = outfile)
            }
            else {
                printgraph(horizontal=TRUE,print=FALSE,file=outfile);
            }
            index1<-index1+1;
        }
        n<-n+1;
    }

    return();
}


##******************************************************************************
ShowPC.func<-function(numclusters)
{
    ## Shows all clusters on the same graph
    n<-1;
    index1<-1;
    npar<-4;
    basetitle<-"GeneShave Principal Component Graphs";
    pclabelfile<-paste(GCoutput,"pclabels.txt",sep="/");
    pclabels<-matrix(scan(pclabelfile,what="",n=numclusters,sep="\n"),
                     numclusters,byrow=FALSE);

    while (n <= numclusters) {
        flag<-n%%npar;
        if (flag == 1) {
            Motif(paste("SPLUS",basetitle,sep=" : "));
            par(mfrow=c(2,2),oma=c(1,1,1,1));
        }

        pcfile<-paste(GCoutput,pclabels[n,1],sep="/");
        PlotPC.func(pcfile);
        clustertitle<-paste("\n","Cluster #",n);
        title(main=clustertitle,cex=1.0);
        if ((n%%npar) == 0 || n == numclusters) {
            maintitle<-paste("\n",basetitle,sep="");
            mtext(maintitle,side=3,outer=TRUE,cex=1.1,font=2);
            ## Save graphs to postscript file
            outfile<-paste(GChome,'output',"GCpcplots",sep="/");
            outfile<-paste(outfile,index1,".ps",sep="");
            if (GCinterpreter == "R") {
			if (psout != 0) dev.copy2eps(file = outfile)
            }
            else {
                printgraph(horizontal=TRUE,print=FALSE,file=outfile);
            }
            index1<-index1+1;
        }
        n<-n+1;
    }

    return();
}


##******************************************************************************
ShowVariance.func<-function(numclusters)
{
    ## Shows all clusters on the same graph
    n<-1;
    index1<-1;
    npar<-4;
    basetitle<-"GeneShave Variance Graphs";
    varlabelsfile<-paste(GCoutput,"vlabels.txt",sep="/");
    varlabels<-matrix(scan(varlabelsfile,what="",
                           n=numclusters,sep="\n"),
                      numclusters,byrow=FALSE);

    while (n <= numclusters) {
        flag<-n%%npar;
        if (flag == 1) {
            Motif(paste("SPLUS",basetitle,sep=" : "));
            par(mfrow=c(2,2),oma=c(1,1,1,1));
        }

        varfile<-paste(GCoutput,varlabels[n,1],sep="/");
        PlotVariance.func(varfile);
        clustertitle<-paste("\n","Cluster #",n);
        title(main=clustertitle,cex=1.0);
        if (GCinterpreter == "R") {
        }
        else {
            key(text=list(c("Real Data","Randomized Data"),col=1:2, adj=1),
                col=c(1,2),type=c("p","p"),points=list(pch=0:1));
	}
        if ((n%%npar) == 0 || n == numclusters) {
            maintitle<-paste("\n",basetitle,sep="");
            mtext(maintitle,side=3,outer=TRUE,cex=1.1,font=2);
            ## Save graphs to postscript file
            outfile<-paste(GChome,'output',"GCvarianceplots",sep="/");
            outfile<-paste(outfile,index1,".ps",sep="");
            if (GCinterpreter == "R") {
		if (psout != 0) dev.copy2eps(file = outfile)
            }
            else {
                printgraph(horizontal=TRUE,print=FALSE,file=outfile);
            }
            index1<-index1+1;
        }
        n<-n+1;
    }

    return();
}


##******************************************************************************
## Calculates the rms value of Signal To Noise Ratio according to
## "Digital Image Processing" by Rafael Gonzalez and Paul Wintz
## Overlays noise on top of X
## Calls GeneShave and displays hierarchical clustering results,
## the gene Expression matrix, clusters and gap and variance graphs
##******************************************************************************
Common.func<-function(X,N,p,nclst,percentToShave,nperm,gapTolerance,noise,df,
                      percentSupervision, clsFilename)
{
    MyData<-CheckNoise.func(X,noise,df);
    GeneExpression<-MyData$Z;
    Noise<-MyData$Y;
    A<-as.vector(matrix(t(X),N*p,1));
    tempA<-A*A;
    ResSumSignal<-sum(tempA);
    B<-as.vector(matrix(t(Noise),N*p,1));
    tempB<-B*B;
    ResSumNoise<-sum(tempB);

    if (noise == 0)
        SNR<- -1.0
    else
        SNR<-ResSumSignal/ResSumNoise;

    flag<-is.matrix(GeneExpression);

    V<-as.vector(matrix(t(GeneExpression),N*p,1));
    is.vector(V);

    verbose<-TRUE;
    generateLabels<-TRUE;

    if (is.null(clsFilename) != TRUE)
        classFileName<-GetAbsPathName.func(clsFilename)
    else
        classFileName<-NULL;

    nclstFound<-0;
    nclstAct<-GeneShave.C(nclst,
                          percentToShave,
                          nperm,
                          gapTolerance,
                          V,
                          N,
                          p,
                          verbose,
                          generateLabels,
                          writeMatrix,
                          percentSupervision,
                          classFileName,
                          nclstFound)[[13]];

    ## calculate SNR
    ## display the original matrix
    matrixgraph<-ShowMatrix.func(GeneExpression,N,p);
    sqrtSNR<-sqrt(SNR);
    if (noise == 0)
        maintitle<-"Gene Expression Matrix, SNR= infinity"
    else
        maintitle<-paste("Gene Expression Matrix, SNR=\t",round(sqrtSNR,4));

    subtitle<-paste("# Genes =",N,"# cell-lines=",p);
    title(main=maintitle,sub=subtitle);
    outfile<-paste(GChome,'output',"GCmatrix.ps",sep="/");
    if (GCinterpreter == "R") {
		if (psout != 0) dev.copy2eps(file = outfile)
    }
    else {
        printgraph(horizontal=TRUE,print=FALSE,file=outfile);
    }

    ## perform hierarchical clustering, if requested
    DisplayHclust.func(GeneExpression,nclstAct);

    ## update user interface
    cat("PK_MONITOR_DISPLAY_RESULTS\n");

    ## display clusters
    Show.func(N,p,nclstAct,2);

    ## display gap
    ShowGap.func(nclstAct);

    ## display variance
    ShowVariance.func(nclstAct);

    ## display principal component
    ShowPC.func(nclstAct);

    par(mfrow=c(1,1));
    cat("PK_MONITOR_APP_PROCESSING_COMPLETE\n");

    return();
}


##*****************************************************************************
DisplayGraphs.func<-function(GCMatrix,nRow,nCol,nClust,flagNoise,SNR)
{
    ## display the original matrix
    matrixgraph<-ShowMatrix.func(GCMatrix,nRow,nCol);
    if (flagNoise < 0) {
        maintitle<-"Observed Gene Expression Matrix";
        subtitle<-paste("# Genes =",nRow,"\t# cell-lines=",nCol);
        title(main=maintitle,sub=subtitle);
	outfile<-paste(GChome,'output',"GCmatrix.ps",sep="/");
	if (GCinterpreter == "R") {
		if (psout != 0) dev.copy2eps(file = outfile)
        }
      
    }
    else if (flagNoise == 0) {
        maintitle<-"Gene Expression Matrix, SNR= infinity";
        subtitle<-paste("# Genes =",nRow,"# cell-lines=",nCol);
        title(main=maintitle,sub=subtitle);
        outfile<-paste(GChome,'output',"GCmatrix.ps",sep="/");
        if (GCinterpreter == "R") {
		if (psout != 0) dev.copy2eps(file = outfile)
        }
        else {
        printgraph(horizontal=TRUE,print=FALSE,file=outfile);
	}
    }
    else {
        sqrtSNR<-sqrt(SNR);
        maintitle<-paste("Gene Expression Matrix, SNR=\t",round(sqrtSNR,4));
        subtitle<-paste("# Genes =",nRow,"# cell-lines=",nCol);
        title(main=maintitle,sub=subtitle);
        outfile<-paste(GChome,'output',"GCmatrix.ps",sep="/");
        if (GCinterpreter == "R") {
		if (psout != 0) dev.copy2eps(file = outfile)
        }
        else {
        printgraph(horizontal=TRUE,print=FALSE,file=outfile);
	}
    }

    ## perform hierarchical clustering, if requested
    DisplayHclust.func(GCMatrix,nclstAct);

    ## update user interface
    cat("PK_MONITOR_DISPLAY_RESULTS\n");

    ## display clusters
    Show.func(nRow,nCol,nClust,2);

    ## display gap
    ShowGap.func(nClust);

    ## display variance
    ShowVariance.func(nClust);

    ## display principal component
    ShowPC.func(nClust);

    par(mfrow=c(1,1));
    cat("PK_MONITOR_APP_PROCESSING_COMPLETE\n");

    return();
}


##******************************************************************************
ErrorHandler.func<-function()
{
    cat("PK_MONITOR_APP_TERMINATED\n");
    cat(paste("PK_MONITOR_ERR_FATAL ",get.message(),"\n",sep=""));
    S <- 'game over';
	write.table(S,paste(GeneclustHome,"error001",sep='/'));
    dump.calls();
}

